@@ -273,15 +273,13 @@ class Deriver(object):
                     if self.data.filter_molecules:
                         if child not in self.data.filter_molecules:
                             good_children.append(child)
-                            self.data.all_good_selfies_children.append(child)
                         else:
                             logger.debug(f"skipping previously seen molecule: {child}")
                     else:
                         good_children.append(child)
-                        self.data.all_good_selfies_children.append(child)
 
         logger.info(f"Generated {len(good_children)} 'good' children.")
-
+        self.data.all_good_selfies_children = good_children
         return good_children, all_filtered_children
 
     def random_selfies(self, *, n_symbols: int = 100, n_molecules: int = 100):
@@ -313,13 +311,12 @@ class Deriver(object):
                     if self.data.filter_molecules:
                         if child not in self.data.filter_molecules:
                             good_children.append(child)
-                            self.data.all_good_selfies_children.append(child)
                         else:
                             logger.debug(f"skipping previously seen molecule: {child}")
                     else:
                         good_children.append(child)
-                        self.data.all_good_selfies_children.append(child)
 
+        self.data.all_good_selfies_children = good_children
         return good_children
 
     def scan_selfies(self):
@@ -355,19 +352,29 @@ class Deriver(object):
                     if self.data.filter_molecules:
                         if child not in self.data.filter_molecules:
                             good_children.append(child)
-                            self.data.all_good_scanner_children.append(child)
                         else:
                             logger.debug(f"skipping previously seen molecule: {child}")
                     else:
                         good_children.append(child)
-                        self.data.all_good_scanner_children.append(child)
 
+        self.data.all_good_scanner_children = good_children
         return good_children, all_filtered_children
 
-    def derive_smiles_gb(self, n_children: int = 100, mut_rate: float = 0.01):
+    def derive_gb(self, n_children: int = 100, mut_rate: float = 0.01, kind='smiles'):
+
+        assert len(self.data.seed_smiles) > 0
         children = []
         good_children = []
-        self.data.all_good_smiles_gb_children = []
+        if kind == 'selfies':
+            self.data.all_good_selfies_gb_children = []
+            crossover_fn = selfies_crossover_gb
+            mutation_fn = selfies_mutate_gb
+        elif kind =='smiles':
+            self.data.all_good_smiles_gb_children = []
+            crossover_fn = crossover_gb
+            mutation_fn = mutate_gb
+        else:
+            raise ValueError('Must specify derivation kind as one of "smiles" or "selfies"')
 
         if self.data.filter:
             filter_params = self.data.filter_params
@@ -377,70 +384,31 @@ class Deriver(object):
                            " in order to use a filter for drug-likeness.")
             filter_params = None
 
-        for _ in range(n_children):
-            parent_a_smiles, parent_b_smiles = random.sample(self.data.seed_smiles, 2)
-            parent_a, parent_b = [Chem.MolFromSmiles(s) for s in (parent_a_smiles, parent_b_smiles)]
-            try:
-                new_child = crossover_gb(parent_a, parent_b)
-                if new_child is not None:
-                    new_child = mutate_gb(new_child, mut_rate)
-                    assert new_child
-                else:
-                    continue
-            except Exception as e:  # pylint: disable=broad-except
-                logger.warning(f"Produced improper SMILES. Ignoring and trying again. Details below:")
-                logger.warning(f"Parents: \n{parent_a_smiles}\n{parent_b_smiles}")
-                logger.warning(e)
-                continue
-            children.append(new_child)
-
-        filtered_children = apply_filter(filter_params,
-                                         children,
-                                         self.data.must_have_patterns,
-                                         self.data.must_not_have_patterns
-                                         )
-        for child in filtered_children:
-            if filtered_children[child]["is_good"]:
-                # check the cache
-                if self.data.filter_molecules:
-                    if child not in self.data.filter_molecules:
-                        good_children.append(child)
-                        self.data.all_good_smiles_gb_children.append(child)
-                    else:
-                        logger.debug(f"skipping previously seen molecule: {child}")
-                else:
-                    good_children.append(child)
-                    self.data.all_good_smiles_gb_children.append(child)
-
-        logger.info(f"Generated {len(good_children)} 'good' children.")
-        return good_children, filtered_children
-
-    def derive_selfies_gb(self, n_children: int = 100, mut_rate: float = 0.01):
-        children = []
-        good_children = []
-        self.data.all_good_selfies_gb_children = []
-
-        if self.data.filter:
-            filter_params = self.data.filter_params
+        parent_a_smiles, parent_b_smiles = (None, None)
+        if len(self.data.seed_smiles) > 1:
+            do_crossover = True
+            new_child = None
         else:
-            logger.warning("Warning: No filter has been set, so all child molecules will be labeled"
-                           " as 'good' regardless of quality. Please call Deriver.set_filter() first"
-                           " in order to use a filter for drug-likeness.")
-            filter_params = None
+            do_crossover = False
+            new_child = self.data.seed_smiles[0]
 
         for _ in range(n_children):
-            parent_a_smiles, parent_b_smiles = random.sample(self.data.seed_smiles, 2)
-            parent_a, parent_b = [Chem.MolFromSmiles(s) for s in (parent_a_smiles, parent_b_smiles)]
             try:
-                new_child = selfies_crossover_gb(parent_a, parent_b)
+                if do_crossover:
+                    parent_a_smiles, parent_b_smiles = random.sample(self.data.seed_smiles, 2)
+                    parent_a, parent_b = [Chem.MolFromSmiles(s) for s in (parent_a_smiles, parent_b_smiles)]
+                    new_child = crossover_fn(parent_a, parent_b)
                 if new_child is not None:
-                    new_child = selfies_mutate_gb(new_child, mut_rate)
-                    assert new_child
+                    mutated_child = mutation_fn(new_child, mut_rate)
+                    assert mutated_child
                 else:
                     continue
             except Exception as e:  # pylint: disable=broad-except
-                logger.warning(f"Produced improper SELFIES. Ignoring and trying again. Details below:")
-                logger.warning(f"Parents: \n{parent_a_smiles}\n{parent_b_smiles}")
+                logger.warning(f"Produced improper {kind.upper()}. Ignoring and trying again. Details below:")
+                if do_crossover:
+                    logger.warning(f"Parents: \n{parent_a_smiles}\n{parent_b_smiles}")
+                if new_child:
+                    logger.warning(f"Pre-mutation child: {new_child}")
                 logger.warning(e)
             children.append(new_child)
 
@@ -455,13 +423,15 @@ class Deriver(object):
                 if self.data.filter_molecules:
                     if child not in self.data.filter_molecules:
                         good_children.append(child)
-                        self.data.all_good_selfies_gb_children.append(child)
                     else:
                         logger.debug(f"skipping previously seen molecule: {child}")
                 else:
                     good_children.append(child)
-                    self.data.all_good_selfies_gb_children.append(child)
 
+        if kind == 'smiles':
+            self.data.all_good_smiles_gb_children = good_children
+        else:
+            self.data.all_good_selfies_gb_children = good_children
         logger.info(f"Generated {len(good_children)} 'good' children.")
         return good_children, filtered_children
 
@@ -599,14 +569,12 @@ class Deriver(object):
                 if self.data.filter_molecules:
                     if child not in self.data.filter_molecules:
                         all_good_children.append(child)
-                        self.data.all_good_brics_children.append(child)
                     else:
                         logger.debug(f"skipping previously seen molecule: {child}")
                 else:
                     # there is no provided list of molecules to skip
                     all_good_children.append(child)
-                    self.data.all_good_brics_children.append(child)
 
         logger.info(f"Generated {len(self.data.all_good_brics_children)} 'good' children.")
-
+        self.data.all_good_brics_children = all_good_children
         return all_good_children, all_filtered_children
